#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <stdarg.h>

#include "grind.h"

extern boolean checki;
extern boolean checkgr;
extern int runsteps;
extern double **rundata;

int Index(char *word, char *sub) {
  /*  return index in w at which string 'sub' begins, or return -1 if sub  */
  /*  is not present */
  int lw, ls;
  int i,s;
  char *pw=word;

  if (word == NULL || sub == NULL)
    return -1;
  lw = strlen(word);
  ls = strlen(sub);

  if (ls>lw)
    return -1;

  for (i=0,s=lw-ls; i<=s; i++)
    if (strncmp(pw++,sub,ls) == 0)
      return i;

  return -1;
}

int nameIndex(char name[]) {
  /*  return index in global var 'all_names' which equals to 'name' */
  /*  if 'name' is not found return -1 */
  int i;

  for (i=0; i<nnames; i++){
    if (0 == strcmp(all_names[i],name))
      return i;
  }
  return -1;
}

int eqf(char argm[], char keyw[]) {
  /*  check two strings and return true iff argm has at least two char's */
  /*  and it is equal or shorter then keyw */
  if (strlen(argm) > strlen(keyw))
    return 0;

  /*  if first char is unequal return */
  if (*argm++ != *keyw++)
    return 0;
  /*  check remaining chars until end of argm argm  */
  while(*argm++ == *keyw++)
    if (*argm == '\0')
      return 1;

  return 0;
}

int lowerword(char word[],char ret[]) {
  /*  convert 'word' to lowercase and store it into 'ret' */
  char *pw=word;
  char *pr=ret;

  while( (*pr++=tolower(*pw++)) != '\0')
    ;
  return 1;
}

int coparr(int n, double yt[], double ys[]) {
  int i;

  for (i=0; i<n; i++)
    yt[i] = ys[i];

  return 1;
}

int chckin(void) {
  int i;

  if (checki)
    return true;
  checki = true;
  for (i=0; i<nvar; i++)
    if (!(vars[i].init)) {
      printf(" Warning: %s not initialized, set to 0.\n",vars[i].name);
      vars[i].init = true;
    }
  for (i=0; i<nexpr; i++)
    exprs[i].init = true;
  for (i=0; i<npar; i++)
    if (!(pars[i].init)) {
      printf(" Warning: %s not initialized, set to 0.\n", pars[i].name);
      pars[i].init = true;
    }
  return 1;
}

int prider(double fstate[]) {
  static double *dy = NULL;
  int i;

  /*  allocate memory for an array which will hold the values of  */
  /*  the derivatives */
  if (dy == NULL) 
    dy = dcalloc(nvar, "prider");

  fcn(fstate,dy);
  printf(sformat,"Population");
  printf(sformat,"State");
  printf(sformat,"Derivative");
  printf("\n");
  for (i=0; i<nvar; i++) {
    printf(sformat, all_names[i]);
    printf(dformat,fstate[i]);
    printf(dformat,dy[i]);
    printf("\n");
  }
  return 1;
}

char *storeStr(const char *str) {
  int len;
  char *str2ret;

  if (NULL == str) {
    gr_warning("returning NULL string\n");
    return NULL;
  }

  len = strlen(str);
  if (!(str2ret = ccalloc(len+1, "'str2ret' in storeStr()")))
    return NULL;
  strcpy(str2ret, str);
  str2ret[len] = '\0';
  return str2ret;
}

char *storeString(char *dest, const char *source) {
  int len;
  if (NULL == source) {
    gr_warning("returning NULL string\n");
    return NULL;
  }
  if (dest) 
    free(dest);
  len = strlen (source);
  if (!(dest = ccalloc(len+1, "'dest' in storeStr()")))
    return NULL;
  strcpy(dest, source);
  if (dest[len] != '\0')
    printf("Strange: %s\n",source);
  return dest;
}

double *dcalloc(int n, char msg[]) {
  double *v;

  if (!(v=(double*)calloc(n,sizeof(double)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}
double **ddcalloc(int n,char msg[]) {
  double **v;

  if (!(v=(double**)calloc(n,sizeof(double*)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}

int *icalloc(int n, char msg[]) {
  int *v;

  if (!(v=(int*)calloc(n,sizeof(int)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}

int **iicalloc(int n, char msg[]) {
  int **v;

  if (!(v=(int**)calloc(n,sizeof(int*)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}

char *ccalloc(int n, char msg[]) {
  char *v;

  if (!(v=(char*)calloc(n,sizeof(char)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}

boolean *bcalloc(int n, char msg[]) {
  int *v;

  if (!(v=(boolean*)calloc(n,sizeof(boolean)))) {
    fprintf(stderr,"Error in memory allocation in %s\n", msg);
    exit (1);
  }
  return v;
}

int sort(int n, double arr[], int ipt[]) {
  int i, j;

  for (i=0; i<n; i++)
    ipt[i] = i;

  for (j=1; j<n; j++) {
    for (i=j-1 ; i>=0; i--) {
      if (arr[ipt[i]] >= arr[j]) {
	ipt[i+1] = j;
	break;
      }
      ipt[i+1] = ipt[i];
    }
    if (i < 0)
      ipt[0] = j;
  }
  return 1;
}

int grepfile(FILE *textfile, char *patt) {
  char line[LINE_LENGTH];
  int c = '\0';
  int i, lim;
  boolean succes = false;

  while (c != EOF) {
    i = 0;
    lim = LINE_LENGTH;
    while (--lim > 0 && (c = getc(textfile)) != EOF && c != '\n')
      line[i++] = c;
    if (c == EOF && lim == (LINE_LENGTH-1))
      return succes;
    line[i] = '\0';
    if (patt) {
      if (strstr(line, patt) != NULL) {
	printf("%s\n",line);
	succes = true;
      }
    }
    else {
      printf("%s\n",line);      
      succes = true;
    }
  }
  return succes;
}

double znormal(double mean, double stdev, boolean pos) {
  static long int idum =-1;
  double g;

  g = gasdev(&idum)*stdev + mean;

  if (pos)
    g = (g<0. ? 0. : g);

  return g;
}

boolean gr_warning(const char * const msg, ...) {
  va_list ap;        /* points to each unnamed arg in turn */

  if (msg == NULL)
    return false;

  printf("Warning: ");
  va_start(ap, msg);  /*  make ap point to first unnamed arg */
  vprintf(msg, ap); /*  let std printf do most of the work */
  va_end(ap); /* clean up */

  return false;
}

int gHelp(char *u, char *d, char *h) {
  if (!strcmp(" ",d)) printf("Usage:   %s\n%s\n",u,h);
  else printf("Usage:   %s\nDefault: %s\n%s\n",u,d,h);
  return 1;
}

int eqArg(int c, char *name, int argc, char *argv[]) {
  if (argc > c && 0 == strcmp(name, argv[c]))
    return 1;
  else
    return 0;
}

int abbrevArg(int c, char *name, int argc, char *argv[]) {
  if (argc > c && eqf(argv[c], name))
    return 1;
  else
    return 0;
}

int isnumeric(int c, int argc, char *argv[]) {
  if (argc > c && (isdigit((int)(argv[c][0])) || '.' == argv[c][0] || 
		   '+' == argv[c][0] || '-' == argv[c][0]))
    return 1;
  else 
    return 0;
}

int mkVarPos(double x[]) {
  int i;

  for (i=0; i<nvar; i++)
    x[i] = (x[i]<0. ? 0. : x[i]);
  return 1;
}

int mkExprPos() {
  int i;

  for (i=0; i<nexpr; i++)
    par_values[i] = (par_values[i]<0. ? 0. : par_values[i]);
  return 1;
}

